--Returns 1 or -1 randomly
function math.randomway()
	return math.random(0,1) * 2 - 1
end

--Returns random range with specified offset between numbers
function math.randomsteps(start, ends, step)
	if type(start) ~= "number" or
	type(ends) ~= "number" or
	type(step) ~= "number" or
	start > ends or
	step <= 0 then
		error("number, number and number expected", 2)
	end

	return start + math.random(0, math.floor((ends - start) / step)) * step
end

--Rounds number
function math.round(num)
	if type(num) ~= "number" then
		error("number expected", 2)
	end

	return math.floor(num + 0.5)
end

--Trims a number
function math.trim(num, starts, ends)
	if type(starts) ~= "number" or
	type(ends) ~= "number" or
	type(num) ~= "number" then
		error("number, number and number expected", 2)
	end

	return (num < starts and starts) or (num > ends and ends) or num
end

--Phytagorean theorem (distance calculation)
function math.distance( w, h, d )
	if type(w) ~= "number" or type(h) ~= "number" or (type(d) ~= "number" and type(d) ~= "nil") then
		error("number, number and optional number expected", 2)
	end

	if d then
		return math.sqrt(w^2 + h^2 + d^2)
	else
		return math.sqrt(w^2 + h^2)
	end
end

--Calculates a point on an ellipse
function math.ellipse(x, y, w, h, a)
	if type(x) ~= "number" or
	type(y) ~= "number" or
	type(w) ~= "number" or
	type(h) ~= "number" or
	type(a) ~= "number" then
		error("number, number, number, number and number expected", 2)
	end

	local ra = a / 180 * math.pi
	local ax = x + w/2 * math.cos(ra)
	local ay = y + h/2 * math.sin(ra)
	return ax, ay
end

--Calculates a point on a bezier curve
function math.bezier(pct, p)
	if type(pct) ~= "number" or type(p) ~= "table" then
		error("number and table expected", 2)
	elseif pct < 0 or pct > 1 or table.maxn(p) < 2 then
		error("invalid arguments", 2)
	end
	for i, v in ipairs(p) do
		if type(v[1]) ~= "number" or type(v[2]) ~= "number" or (type(v[3]) ~= "number" and type(v[3]) ~= "nil") then
			error("invalid table content", 2)
		end
	end

	--Factorial
	local function fac(n)
		local k = 1
		if n > 1 then
			for i=2, n do
				k = k * i
			end
		end
		return k
	end
	--Binomial coefficient
	local function bin(i, n)
		return fac(n) / (fac(i) * fac(n-i))
	end
	--Bernstein polynom
	local function bernstein(pct, i, n)
		return bin(i, n) * math.pow(pct,i) * math.pow((1 - pct), n - i)
	end
	--Calculate coordinate
	local point = {0, 0, 0}
	local n = table.maxn(p) - 1
	for i=0, n do
		local bern = bernstein(pct, i, n)
		point[1] = point[1] + p[i+1][1] * bern
		point[2] = point[2] + p[i+1][2] * bern
		point[3] = point[3] + (p[i+1][3] or 0) * bern
	end
	return point
end

--Creates a matrix object
function math.matrix()
	local matrix = {}
	--Resets matrix
	function matrix:reset()
		self.matrix = {
			1, 0, 0, 0,
			0, 1, 0, 0,
			0, 0, 1, 0,
			0, 0, 0, 1
		}
	end

	--Matrix origin point
	function matrix:set_origin(point)
		if type(point) ~= "table" or #point < 3 then
			error("valid table expected", 2)
		end
		for i = 1, 3 do
			if type(point[i]) ~= "number" then
				error("numbers in table expected", 2)
			end
		end

		self.origin[1] = point[1]
		self.origin[2] = point[2]
		self.origin[3] = point[3]
	end

	--Transforms point
	function matrix:transform(point)
		if type(point) ~= "table" or #point < 3 then
			error("valid table expected", 2)
		end
		for i = 1, 3 do
			if type(point[i]) ~= "number" then
				error("numbers in table expected", 2)
			end
		end

		local p = {point[1] - self.origin[1], point[2] - self.origin[2], point[3] - self.origin[3]}
		local w = self.matrix[4] * p[1] + self.matrix[8] * p[2] + self.matrix[12] * p[3] + self.matrix[16]

		point[1] = (self.matrix[1] * p[1] + self.matrix[5] * p[2] + self.matrix[9] * p[3] + self.matrix[13]) / w + self.origin[1]
		point[2] = (self.matrix[2] * p[1] + self.matrix[6] * p[2] + self.matrix[10] * p[3] + self.matrix[14]) / w + self.origin[2]
		point[3] = (self.matrix[3] * p[1] + self.matrix[7] * p[2] + self.matrix[11] * p[3] + self.matrix[15]) / w + self.origin[3]
	end

	--Multiplies matrix
	function matrix:mult(mat2)
		if type(mat2) ~= "table" or #mat2 ~= 16 then
			error("valid table expected", 2)
		end
		for i, v in ipairs(mat2) do
			if type(v) ~= "number" then
				error("numbers in table expected", 2)
			end
		end

		local new_matrix = {
			0, 0, 0, 0,
			0, 0, 0, 0,
			0, 0, 0, 0,
			0, 0, 0, 0
		}
		for i = 1, 16 do
			local h_start = 1 + math.mod(i-1, 4)
			local v_start = 1 + math.floor( (i-1)/4 ) * 4
			for i2 = 0, 3 do
				new_matrix[i] = new_matrix[i] + self.matrix[h_start + i2 * 4] * mat2[v_start + i2]
			end
		end
		self.matrix = new_matrix
	end

	--Translate matrix
	function matrix:translate(x, y, z)
		if type(x) ~= "number" or type(y) ~= "number" or type(z) ~= "number" then
			error("number, number and number expected", 2)
		end

		matrix.mult(self, {
			1, 0, 0, 0,
			0, 1, 0, 0,
			0, 0, 1, 0,
			x, y, z, 1
		})
	end

	--Scale matrix
	function matrix:scale(x, y, z)
		if type(x) ~= "number" or type(y) ~= "number" or type(z) ~= "number" then
			error("number, number and number expected", 2)
		end

		matrix.mult(self, {
			x, 0, 0, 0,
			0, y, 0, 0,
			0, 0, z, 0,
			0, 0, 0, 1
		})
	end

	--Rotate matrix
	function matrix:rotate(typ, angle)
		if type(typ) ~= "string" or type(angle) ~= "number" then
			error("string and number expected", 2)
		end

		local angle = angle / 180 * math.pi
		local function s()
			return math.sin(angle)
		end
		local function c()
			return math.cos(angle)
		end

		if typ == "x" then
			matrix.mult(self, {
				1, 0, 0, 0,
				0, c(), s(), 0,
				0, -s(), c(), 0,
				0, 0, 0, 1
			})
		elseif typ == "y" then
			matrix.mult(self, {
				c(), 0, -s(), 0,
				0, 1, 0, 0,
				s(), 0, c(), 0,
				0, 0, 0, 1
			})
		elseif typ == "z" then
			matrix.mult(self, {
				c(), s(), 0, 0,
				-s(), c(), 0, 0,
				0, 0, 1, 0,
				0, 0, 0, 1
			})
		else
			error("invalid string", 2)
		end
	end

	--Create matrix object
	local obj  = {
		matrix =
		{
			1, 0, 0, 0,
			0, 1, 0, 0,
			0, 0, 1, 0,
			0, 0, 0, 1
		},
		origin = {
			0, 0, 0
		}
	}
	setmetatable(obj, matrix)
	matrix.__index = matrix
	return obj
end
