--Creates an outline path object
function GetPath(shape, style)
	if type(shape) ~= "string" or (type(style) ~= "nil" and type(style) ~= "table") then
		error("string and optional table expected", 2)
	end

	local path = {}
	--Splits path lines in shorter segments
	function path:split(len)
		if type(len) ~= "number" and len > 0 then
			error("valid number expected", 2)
		end

		local path = {}
		for pi, point in ipairs(self.path) do
			if point.type ~= "m" and pi > 1 then
				local pre_point = self.path[pi-1]
				local vec = {point[1] - pre_point[1], point[2] - pre_point[2], point[3] - pre_point[3]}
				local distance = math.sqrt(vec[1]^2 + vec[2]^2 + vec[3]^2)
				for i = len * self.scale, distance, len * self.scale do
					local pct = i / distance
					table.insert(path, {pre_point[1] + pct * vec[1], pre_point[2] + pct * vec[2], pre_point[3] + pct * vec[3], type = point.type})
				end
			end
			table.insert(path, point)
		end
		self.path = path
	end

	--Calls a filter function for every path point
	function path:filter(func)
		if type(func) ~= "function" then
			error("function expected", 2)
		end

		for i = 1, #self.path do
			func(self.path[i], i, #self.path)
		end
	end

	--Moves the path
	function path:move(x, y, z)
		if type(x) ~= "number" or type(y) ~= "number" or (type(z) ~= "number" and type(z) ~= "nil") then
			error("number, number and optional number expected", 2)
		end

		local function path_movement(p)
			p[1] = p[1] + x * self.scale
			p[2] = p[2] + y * self.scale
			p[3] = p[3] + z * self.scale
		end
		path.filter(self, path_movement)
	end

	--Returns bounding box of path
	function path:getbound()
		local min_x, min_y, min_z, max_x, max_y, max_z
		local function getbound(point)
			min_x = min_x and math.min(min_x, point[1]/self.scale) or point[1]/self.scale
			min_y = min_y and math.min(min_y, point[2]/self.scale) or point[2]/self.scale
			min_z = min_z and math.min(min_z, point[3]/self.scale) or point[3]/self.scale
			max_x = max_x and math.max(max_x, point[1]/self.scale) or point[1]/self.scale
			max_y = max_y and math.max(max_y, point[2]/self.scale) or point[2]/self.scale
			max_z = max_z and math.max(max_z, point[3]/self.scale) or point[3]/self.scale
		end
		path.filter(self, getbound)
		if min_x == nil then
			min_x = 0
			min_y = 0
			min_z = 0
			max_x = 0
			max_y = 0
			max_z = 0
		end
		return min_x, min_y, min_z, max_x, max_y, min_z
	end

	--Returns path as shape command
	function path:tostring()
		local shape, last_type = "", ""
		for pi, point in ipairs(self.path) do
			if point.type ~= last_type then
				shape = shape .. point.type .. " "
				last_type = point.type
			end
			shape = shape .. math.floor(point[1]+0.5) .. " " .. math.floor(point[2]+0.5)
			if pi < #self.path then
				shape = shape .. " "
			end
		end
		return shape
	end

	--Returns path as pixels
	function path:topixels()
		local pixels
		local min_x, min_y = 0, 0

		local function minmax(point)
			min_x, min_y = math.min(min_x, point[1]), math.min(min_y, point[2])
		end
		path.filter(self, minmax)

		if min_x < 0 or min_y < 0 then
			local function pos_shifter(point)
				point[1] = point[1] - min_x
				point[2] = point[2] - min_y
			end
			path.filter(self, pos_shifter)
			pixels = GetPixelsW(path.tostring(self))
			for pi, pixel in ipairs(pixels) do
				pixel.x = pixel.x + min_x / self.scale
				pixel.y = pixel.y + min_y / self.scale
			end
		else
			pixels = GetPixelsW(path.tostring(self))
		end

		return pixels
	end

	--Converts path to the outline path of the outline
	function path:tooutline(size)
		if type(size) ~= "number" or size <= 0 then
			error("valid number expected", 2)
		end

		--Help functions
		local function vec_length(vec)
			return math.sqrt(vec[1]^2 + vec[2]^2 + vec[3]^2)
		end

		local function vec_sizer(vec, size)
			local len = vec_length(vec)
			if len == 0 then
				return {0, 0, 0}
			else
				vec[1] = vec[1] / len * size
				vec[2] = vec[2] / len * size
				vec[3] = vec[3] / len * size
				return vec
			end
		end

		local vec2 = {0, 0, 1}
		local function calc_ortho_vec(p1, p2)
			local vec1 = {p2[1]-p1[1], p2[2]-p1[2], p2[3]-p1[3]}
			return {
				vec1[2] * vec2[3] - vec1[3] * vec2[2],
				vec1[3] * vec2[1] - vec1[1] * vec2[3],
				vec1[1] * vec2[2] - vec1[2] * vec2[1]
			}
		end

		--Path tables
		local paths = {}
		local stroke_paths = {{},{}}	--inner + outer
		local temp_path = {}

		--Path parts
		for pi = 1, #self.path do
			table.insert(temp_path, self.path[pi])
			if pi == #self.path or self.path[pi+1].type == "m" then
				table.insert(paths, temp_path)
				temp_path = {}
			end
		end

		--Erase double-points
		for path_i, path in ipairs(paths) do
			local pi = 1
			while pi <= #path do
				local point = path[pi]
				local post_point
				if pi == #path then
					post_point = path[1]
				else
					post_point = path[pi+1]
				end
				if point[1] == post_point[1] and point[2] == post_point[2] and point[3] == post_point[3] then
					table.remove(path, pi)
				else
					pi = pi + 1
				end
			end
		end

		--Stroke paths
		for i = 1, 2 do
			for path_i, path in ipairs(paths) do
				local loop_begin, loop_end, loop_steps
				if i == 1 then
					loop_begin, loop_end, loop_steps = #path, 1, -1
				else
					loop_begin, loop_end, loop_steps = 1, #path, 1
				end
				for pi = loop_begin, loop_end, loop_steps do
					local point = path[pi]
					local pre_point, post_point
					if i == 1 then
						if pi == 1 then
							pre_point = path[pi+1]
							post_point = path[#path]
						elseif pi == #path then
							pre_point = path[1]
							post_point = path[pi-1]
						else
							pre_point = path[pi+1]
							post_point = path[pi-1]
						end
					else
						if pi == 1 then
							pre_point = path[#path]
							post_point = path[pi+1]
						elseif pi == #path then
							pre_point = path[pi-1]
							post_point = path[1]
						else
							pre_point = path[pi-1]
							post_point = path[pi+1]
						end
					end
					local o_vec1 = vec_sizer( calc_ortho_vec(pre_point, point), size*self.scale)
					local o_vec2 = vec_sizer( calc_ortho_vec(point, post_point), size*self.scale)
					local o_vec12 = {
										o_vec2[1] - o_vec1[1],
										o_vec2[2] - o_vec1[2],
										o_vec2[3] - o_vec1[3],
									}
					local len = vec_length(o_vec12)
					if len == 0 then
						table.insert(temp_path, {
										math.floor(point[1] + o_vec1[1]),
										math.floor(point[2] + o_vec1[2]),
										math.floor(point[3] + o_vec1[3]),
										type = (#temp_path == 0 and "m" or "l")
										})
					else
						for o = 0, len, 2 do
							local pct = o / len
							local o_vecp = vec_sizer( {
														o_vec1[1]+o_vec12[1]*pct,
														o_vec1[2]+o_vec12[2]*pct,
														o_vec1[3]+o_vec12[3]*pct,
													}, size*self.scale)
							table.insert(temp_path, {
											math.floor(point[1] + o_vecp[1]),
											math.floor(point[2] + o_vecp[2]),
											math.floor(point[3] + o_vecp[3]),
											type = (#temp_path == 0 and "m" or "l")
											})
						end
					end
				end
				table.insert(stroke_paths[i], temp_path)
				temp_path = {}
			end
		end

		--Concatenate stroke inner and outer paths to new final path
		self.path = {}
		for path_i = 1, #stroke_paths[1] do
			local inner_path = stroke_paths[1][path_i]
			local outer_path = stroke_paths[2][path_i]
			local last_inner = {inner_path[1][1], inner_path[1][2], inner_path[1][3], type = "l"}	--add last line to close shape
			local last_outer = {outer_path[1][1], outer_path[1][2], outer_path[1][3], type = "l"}
			table.insert(inner_path, last_inner)
			table.insert(outer_path, last_outer)
			for pi, point in ipairs(inner_path) do
				table.insert(self.path, point)
			end
			for pi, point in ipairs(outer_path) do
				table.insert(self.path, point)
			end
		end
	end

	--Create path object
	local obj
	if style then
		obj = {path = GetPathW(shape, style), scale = 8}
	else
		obj = {path = GetPathW(shape), scale = 8}
	end
	setmetatable(obj, path)
	path.__index = path
	return obj
end
