--Converts a RGB value to an ASS string
function RGB_to_ASS(r,g,b)
	if type(r) ~= "number" or type(g) ~= "number" or type(b) ~= "number" then
		error("number, number and number expected", 2)
	end

	return string.format("&H%02x%02x%02x&", b, g, r)
end

--Converts an ASS string to a RGB value
function ASS_to_RGB(str)
	if type(str) ~= "string" then
		error("string expected", 2)
	end

	local b, g, r = str:match("&H(%x%x)(%x%x)(%x%x)&")
	if not (b and g and r) then
		return false
	else
		return tonumber(r, 16), tonumber(g, 16), tonumber(b, 16)
	end
end

--Converts an alpha value to an ASS string
function A_to_ASS(a)
	if type(a) ~= "number" then
		error("number expected", 2)
	end

	return string.format("&H%02x&", 255-a)
end

--Converts an ASS string to an alpha value
function ASS_to_A(str)
	if type(str) ~= "string" then
		error("string expected", 2)
	end

	local a = str:match("&H(%x%x)&")
	if not a then
		return false
	else
		return 255-tonumber(a, 16)
	end
end

--Interpolates an alpha, a color or a number value
function Interpolate(pct, val1, val2, calc)
	if type(pct) ~= "number" or
	not (
	(type(val1) == "number" and type(val2) == "number") or
	(type(val1) == "string" and type(val2) == "string")
	) or
	(type(calc) ~= "string" and type(calc) ~= "nil") then
		error("number and 2 numbers or 2 strings and optional string expected", 2)
	end

	local pct = pct
	if calc ~= nil then
		if calc == "curve_up" then
			pct = math.sin(pct * math.pi)
		elseif calc == "curve_down" then
			pct = 1 - math.sin(pct * math.pi)
		elseif calc == "root" then
			pct = math.sqrt(pct)
		elseif calc == "square" then
			pct = pct^2
		else
			error("valid last string expected", 2)
		end
	end

	if type(val1) == "number" then
		return val1 + (val2 - val1) * pct
	else
		local r1, g1, b1 = ASS_to_RGB(val1)
		local r2, g2, b2 = ASS_to_RGB(val2)
		if b1 and b2 then
			local r, g, b
			r = (r2 - r1) * pct + r1
			g = (g2 - g1) * pct + g1
			b = (b2 - b1) * pct + b1
			return RGB_to_ASS(r, g, b)
		end
		local a1 = ASS_to_A(val1)
		local a2 = ASS_to_A(val2)
		if a1 and a2 then
			local a = (a2 - a1) * pct + a1
			return A_to_ASS(a)
		end
		error("invalid arguments", 2)
	end
end

--Iterator through frame times
function frames(starts, ends, frame_time)
	if type(starts) ~= "number" or type(ends) ~= "number" or type(frame_time) ~= "number" or frame_time <= 0 then
		error("number, number and number expected", 2)
	end

	local cur_start_time, i, n = starts, 0, math.ceil((ends - starts) / frame_time)
	--Iterator function
	local function next_frame()
		if cur_start_time >= ends then
			return nil
		end
		local return_start_time = cur_start_time
		local return_end_time = return_start_time + frame_time <= ends and return_start_time + frame_time or ends
		cur_start_time = return_end_time
		i = i + 1
		return return_start_time, return_end_time, i, n
	end
	return next_frame
end



--Distributes values
function Distributor(t)
	if type(t) ~= "table" or table.maxn(t) < 1 then
		error("table expected (not empty)", 2)
	end

	local func_t = {}
	local index = 1
	function func_t:get()
		if index > #self.t then index = 1 end
		local val = self.t[index]
		index = index + 1
		return val
	end

	local obj  = {t = t}
	setmetatable(obj, func_t)
	func_t.__index = func_t
	return obj
end
